local function buildModels(image, frameNum, frameWidth)

    -- Les normals dans verts sont toutes à `0,0,1`
    -- Cela signifie quelles pointent toutes vers l'avant du model

    local models = {}

    for i = 1, frameNum do

        local u = ( i - 1 ) * frameWidth
        local nu = i * frameWidth

        local verts = {
            { -.5,.5,0, u,1, 0,0,1 },
            { .5,.5,0, nu,1, 0,0,1 },
            { .5,-.5,0, nu,0, 0,0,1 },
            { -.5,-.5,0, u,0, 0,0,1 },
            { -.5,.5,0, u,1, 0,0,1 },
            { .5,-.5,0, nu,0, 0,0,1 }
        }

        models[i] = g3d.newModel(verts, image)

    end

    return models

end

local Animation = Class {

    init = function (self, image, playTime)

        self.image = image
        self.playTime = playTime or 1

        self.timer = 0
        self.looping = true

        self.frameNum = image:getWidth() / image:getHeight()

        self.frameWidth = 1 / self.frameNum

        self.models = buildModels(self.image, self.frameNum, self.frameWidth)
        self.model = self.models[1]

    end;


    -- Transformation part --

    setTransform = function (self, translation, rotation, scale)
        for _, model in ipairs(self.models) do
            model:setTransform(translation, rotation, scale)
        end
    end;

    setTranslation = function (self, tx, ty, tz)
        for _, model in ipairs(self.models) do
            model:setTranslation(tx,ty,tz)
        end
    end;

    setRotation = function (self, rx, ry, rz)
        for _, model in ipairs(self.models) do
            model:setRotation(rx,ry,rz)
        end
    end;

    setAxisAngleRotation = function (self, x, y, z, angle)
        for _, model in ipairs(self.models) do
            model:setAxisAngleRotation(x,y,z,angle)
        end
    end;

    setQuaternionRotation = function (self, x, y, z, w)
        for _, model in ipairs(self.models) do
            model:setQuaternionRotation(x,y,z,w)
        end
    end;

    setScale = function (self, sx, sy, sz)
        for _, model in ipairs(self.models) do
            model:setScale(sx,sy,sz)
        end
    end;


    -- Collision part --

    rayIntersection = function (self, ...)
        return self.model:rayIntersection(...)
    end;

    isPointInside = function (self, ...)
        return self.model:isPointInside(...)
    end;

    sphereIntersection = function (self, ...)
        return self.model:sphereIntersection(...)
    end;

    closestPoint = function (self, ...)
        return self.model:closestPoint(...)
    end;

    capsuleIntersection = function (self, ...)
        return self.model:capsuleIntersection(...)
    end;


    -- Animation part --

    setLooping = function (self, looping)
        self.looping = looping
    end;

    setRotToCam = function (self, cam)

        local dx = cam.position[1] - self.model.translation[1]
        local dy = cam.position[2] - self.model.translation[2]

        local D90 = math.pi/2
        local angle = math.atan2(dy, dx)+D90
        self.model:setRotation(D90,math.pi,angle)

    end;

    update = function (self, hdt)    -- hdt = math.min(dt, 1/15)

        self.timer = self.timer + hdt

        if self.looping then
            self.timer = self.timer % self.playTime
        end

        self.timer = math.min(self.playTime - .000001, self.timer)

        local index = math.floor(self.timer / self.playTime * self.frameNum) + 1

        self.model = self.models[index]

    end;

    draw = function (self, shader)
        self.model:draw(shader)
    end

}

return Animation